@@ -13,7 +13,7 @@
  *   ORFEUS/EC-Project MEREDIAN
  *   IRIS Data Management Center
  *
- * modified: 2009.111
+ * modified: 2012.114
  ***************************************************************************/
 #include <stdio.h>
 #include <stdlib.h>
@@ -83,11 +83,12 @@ msr_unpack ( char *record, int reclen, MSRecord **ppmsr,
   char srcname[50];
   
   /* For blockette parsing */
-  BlktLink *blkt_link;
+  BlktLink *blkt_link = 0;
   uint16_t blkt_type;
   uint16_t next_blkt;
   uint32_t blkt_offset;
   uint32_t blkt_length;
+  int blkt_count = 0;
   
   if ( ! ppmsr )
     {
@@ -145,7 +146,7 @@ msr_unpack ( char *record, int reclen, MSRecord **ppmsr,
   
   /* Check to see if byte swapping is needed by testing the year */
   if ( (msr->fsdh->start_time.year < 1920) ||
-       (msr->fsdh->start_time.year > 2020) )
+       (msr->fsdh->start_time.year > 2050) )
     headerswapflag = dataswapflag = 1;
   
   /* Check if byte order is forced */
@@ -172,13 +173,13 @@ msr_unpack ( char *record, int reclen, MSRecord **ppmsr,
     }
   
   /* Populate some of the common header fields */
-  ms_strncpclean (sequence_number, msr->fsdh->sequence_number, 6);
+  strncpy (sequence_number, msr->fsdh->sequence_number, 6);
   msr->sequence_number = (int32_t) strtol (sequence_number, NULL, 10);
   msr->dataquality = msr->fsdh->dataquality;
-  ms_strncpclean (msr->network, msr->fsdh->network, 2);
-  ms_strncpclean (msr->station, msr->fsdh->station, 5);
-  ms_strncpclean (msr->location, msr->fsdh->location, 2);
-  ms_strncpclean (msr->channel, msr->fsdh->channel, 3);
+  ms_strncpcleantail (msr->network, msr->fsdh->network, 2);
+  ms_strncpcleantail (msr->station, msr->fsdh->station, 5);
+  ms_strncpcleantail (msr->location, msr->fsdh->location, 2);
+  ms_strncpcleantail (msr->channel, msr->fsdh->channel, 3);
   msr->samplecnt = msr->fsdh->numsamples;
   
   /* Generate source name for MSRecord */
@@ -206,7 +207,7 @@ msr_unpack ( char *record, int reclen, MSRecord **ppmsr,
   blkt_offset = msr->fsdh->blockette_offset;
   
   while ((blkt_offset != 0) &&
-	 (blkt_offset < reclen) &&
+	 ((int)blkt_offset < reclen) &&
 	 (blkt_offset < MAXRECLEN))
     {
       /* Every blockette has a similar 4 byte header: type and next */
@@ -234,7 +235,7 @@ msr_unpack ( char *record, int reclen, MSRecord **ppmsr,
 	}
       
       /* Make sure blockette is contained within the msrecord buffer */
-      if ( (blkt_offset - 4 + blkt_length) > reclen )
+      if ( (int)(blkt_offset - 4 + blkt_length) > reclen )
 	{
 	  ms_log (2, "msr_unpack(%s): Blockette %d extends beyond record size, truncated?\n",
 		  UNPACK_SRCNAME, blkt_type);
@@ -251,6 +252,7 @@ msr_unpack ( char *record, int reclen, MSRecord **ppmsr,
 	  if ( ! blkt_link )
 	    break;
 	  
+	  blkt_link->blktoffset = blkt_offset - 4;
 	  blkt_link->next_blkt = next_blkt;
 	  
 	  blkt_100 = (struct blkt_100_s *) blkt_link->blktdata;
@@ -273,6 +275,7 @@ msr_unpack ( char *record, int reclen, MSRecord **ppmsr,
 	  if ( ! blkt_link )
 	    break;
 	  
+	  blkt_link->blktoffset = blkt_offset - 4;
 	  blkt_link->next_blkt = next_blkt;
 	  
 	  blkt_200 = (struct blkt_200_s *) blkt_link->blktdata;
@@ -296,6 +299,7 @@ msr_unpack ( char *record, int reclen, MSRecord **ppmsr,
 	  if ( ! blkt_link )
 	    break;
 	  
+	  blkt_link->blktoffset = blkt_offset - 4;
 	  blkt_link->next_blkt = next_blkt;
 	  
 	  blkt_201 = (struct blkt_201_s *) blkt_link->blktdata;
@@ -319,6 +323,7 @@ msr_unpack ( char *record, int reclen, MSRecord **ppmsr,
 	  if ( ! blkt_link )
 	    break;
 	  
+	  blkt_link->blktoffset = blkt_offset - 4;
 	  blkt_link->next_blkt = next_blkt;
 	  
 	  blkt_300 = (struct blkt_300_s *) blkt_link->blktdata;
@@ -343,6 +348,7 @@ msr_unpack ( char *record, int reclen, MSRecord **ppmsr,
 	  if ( ! blkt_link )
 	    break;
 	  
+	  blkt_link->blktoffset = blkt_offset - 4;
 	  blkt_link->next_blkt = next_blkt;
 	  
 	  blkt_310 = (struct blkt_310_s *) blkt_link->blktdata;
@@ -367,6 +373,7 @@ msr_unpack ( char *record, int reclen, MSRecord **ppmsr,
 	  if ( ! blkt_link )
 	    break;
 	  
+	  blkt_link->blktoffset = blkt_offset - 4;
 	  blkt_link->next_blkt = next_blkt;
 	  
 	  blkt_320 = (struct blkt_320_s *) blkt_link->blktdata;
@@ -390,6 +397,7 @@ msr_unpack ( char *record, int reclen, MSRecord **ppmsr,
 	  if ( ! blkt_link )
 	    break;
 	  
+	  blkt_link->blktoffset = blkt_offset - 4;
 	  blkt_link->next_blkt = next_blkt;
 	  
 	  blkt_390 = (struct blkt_390_s *) blkt_link->blktdata;
@@ -412,6 +420,7 @@ msr_unpack ( char *record, int reclen, MSRecord **ppmsr,
 	  if ( ! blkt_link )
 	    break;
 	  
+	  blkt_link->blktoffset = blkt_offset - 4;
 	  blkt_link->next_blkt = next_blkt;
 	  
 	  blkt_395 = (struct blkt_395_s *) blkt_link->blktdata;
@@ -432,6 +441,7 @@ msr_unpack ( char *record, int reclen, MSRecord **ppmsr,
 	  if ( ! blkt_link )
 	    break;
 	  
+	  blkt_link->blktoffset = blkt_offset - 4;
 	  blkt_link->next_blkt = next_blkt;
 	  
 	  blkt_400 = (struct blkt_400_s *) blkt_link->blktdata;
@@ -454,6 +464,7 @@ msr_unpack ( char *record, int reclen, MSRecord **ppmsr,
 	  if ( ! blkt_link )
 	    break;
 	  
+	  blkt_link->blktoffset = blkt_offset - 4;
 	  blkt_link->next_blkt = next_blkt;
 	  
 	  blkt_405 = (struct blkt_405_s *) blkt_link->blktdata;
@@ -480,6 +491,7 @@ msr_unpack ( char *record, int reclen, MSRecord **ppmsr,
 	  if ( ! blkt_link )
 	    break;
 	  
+	  blkt_link->blktoffset = blkt_offset - 4;
 	  blkt_link->next_blkt = next_blkt;
 	  
 	  blkt_500 = (struct blkt_500_s *) blkt_link->blktdata;
@@ -502,6 +514,7 @@ msr_unpack ( char *record, int reclen, MSRecord **ppmsr,
 	  if ( ! blkt_link )
 	    break;
 	  
+	  blkt_link->blktoffset = blkt_offset - 4;
 	  blkt_link->next_blkt = next_blkt;
 
 	  blkt_1000 = (struct blkt_1000_s *) blkt_link->blktdata;
@@ -528,6 +541,7 @@ msr_unpack ( char *record, int reclen, MSRecord **ppmsr,
 	  if ( ! blkt_link )
 	    break;
 	  
+	  blkt_link->blktoffset = blkt_offset - 4;
 	  blkt_link->next_blkt = next_blkt;
 	}
       
@@ -548,6 +562,7 @@ msr_unpack ( char *record, int reclen, MSRecord **ppmsr,
 	  if ( ! blkt_link )
 	    break;
 	  
+	  blkt_link->blktoffset = blkt_offset - 4;
 	  blkt_link->next_blkt = next_blkt;
 	  
 	  blkt_2000 = (struct blkt_2000_s *) blkt_link->blktdata;
@@ -571,15 +586,16 @@ msr_unpack ( char *record, int reclen, MSRecord **ppmsr,
 	      if ( ! blkt_link )
 		break;
 	      
+	      blkt_link->blktoffset = blkt_offset - 4;
 	      blkt_link->next_blkt = next_blkt;
 	    }
 	}
       
-      /* Check that the offset increases */
-      if ( next_blkt && next_blkt <= blkt_offset )
+      /* Check that the next blockette offset is beyond the current blockette */
+      if ( next_blkt && next_blkt < (blkt_offset + blkt_length - 4) )
 	{
-	  ms_log (2, "msr_unpack(%s): Offset to next blockette (%d) from type %d did not increase\n",
-		  UNPACK_SRCNAME, next_blkt, blkt_type);
+	  ms_log (2, "msr_unpack(%s): Offset to next blockette (%d) is within current blockette ending at byte %d\n",
+		  UNPACK_SRCNAME, next_blkt, (blkt_offset + blkt_length - 4));
 	  
 	  blkt_offset = 0;
 	}
@@ -595,8 +611,11 @@ msr_unpack ( char *record, int reclen, MSRecord **ppmsr,
 	{
 	  blkt_offset = next_blkt;
 	}
+      
+      blkt_count++;
     }  /* End of while looping through blockettes */
   
+  /* Check for a Blockette 1000 */
   if ( msr->Blkt1000 == 0 )
     {
       if ( verbose > 1 )
@@ -605,6 +624,20 @@ msr_unpack ( char *record, int reclen, MSRecord **ppmsr,
 	}
     }
   
+  /* Check that the data offset is after the blockette chain */
+  if ( blkt_link && msr->fsdh->numsamples && msr->fsdh->data_offset < (blkt_link->blktoffset + blkt_link->blktdatalen + 4) )
+    {
+      ms_log (1, "%s: Warning: Data offset in fixed header (%d) is within the blockette chain ending at %d\n",
+	      UNPACK_SRCNAME, msr->fsdh->data_offset, (blkt_link->blktoffset + blkt_link->blktdatalen + 4));
+    }
+  
+  /* Check that the blockette count matches the number parsed */
+  if ( msr->fsdh->numblockettes != blkt_count )
+    {
+      ms_log (1, "%s: Warning: Number of blockettes in fixed header (%d) does not match the number parsed (%d)\n",
+	      UNPACK_SRCNAME, msr->fsdh->numblockettes, blkt_count);
+    }
+  
   /* Populate remaining common header fields */
   msr->starttime = msr_starttime (msr);
   msr->samprate = msr_samprate (msr);
@@ -646,9 +679,11 @@ msr_unpack ( char *record, int reclen, MSRecord **ppmsr,
 	{
 	  dswapflag = 0;
 	  
+	  /* If BE host and LE data need swapping */
 	  if ( bigendianhost && msr->byteorder == 0 )
 	    dswapflag = 1;
-	  else if ( !bigendianhost && msr->byteorder == 1 )
+	  /* If LE host and BE data (or bad byte order value) need swapping */
+	  else if ( !bigendianhost && msr->byteorder > 0 )
 	    dswapflag = 1;
 	}
       else if ( unpackdatabyteorder >= 0 )
@@ -739,7 +774,7 @@ msr_unpack_data ( MSRecord *msr, int swapflag, int verbose )
     }
   
   /* Calculate buffer size needed for unpacked samples */
-  unpacksize = msr->samplecnt * samplesize;
+  unpacksize = (int) msr->samplecnt * samplesize;
   
   /* (Re)Allocate space for the unpacked data */
   if ( unpacksize > 0 )
@@ -765,8 +800,8 @@ msr_unpack_data ( MSRecord *msr, int swapflag, int verbose )
   dbuf = msr->record + msr->fsdh->data_offset;
   
   if ( verbose > 2 )
-    ms_log (1, "%s: Unpacking %d samples\n",
-	    UNPACK_SRCNAME, msr->samplecnt);
+    ms_log (1, "%s: Unpacking %lld samples\n",
+	    UNPACK_SRCNAME, (long long int)msr->samplecnt);
   
   /* Decide if this is a encoding that we can decode */
   switch (msr->encoding)
@@ -776,7 +811,7 @@ msr_unpack_data ( MSRecord *msr, int swapflag, int verbose )
       if ( verbose > 1 )
 	ms_log (1, "%s: Found ASCII data\n", UNPACK_SRCNAME);
       
-      nsamples = msr->samplecnt;
+      nsamples = (int)msr->samplecnt;
       memcpy (msr->datasamples, dbuf, nsamples);
       msr->sampletype = 'a';      
       break;
@@ -785,8 +820,8 @@ msr_unpack_data ( MSRecord *msr, int swapflag, int verbose )
       if ( verbose > 1 )
 	ms_log (1, "%s: Unpacking INT-16 data samples\n", UNPACK_SRCNAME);
       
-      nsamples = msr_unpack_int_16 ((int16_t *)dbuf, msr->samplecnt,
-				    msr->samplecnt, msr->datasamples,
+      nsamples = msr_unpack_int_16 ((int16_t *)dbuf, (int)msr->samplecnt,
+				    (int)msr->samplecnt, msr->datasamples,
 				    swapflag);
       msr->sampletype = 'i';
       break;
@@ -795,8 +830,8 @@ msr_unpack_data ( MSRecord *msr, int swapflag, int verbose )
       if ( verbose > 1 )
 	ms_log (1, "%s: Unpacking INT-32 data samples\n", UNPACK_SRCNAME);
       
-      nsamples = msr_unpack_int_32 ((int32_t *)dbuf, msr->samplecnt,
-				    msr->samplecnt, msr->datasamples,
+      nsamples = msr_unpack_int_32 ((int32_t *)dbuf, (int)msr->samplecnt,
+				    (int)msr->samplecnt, msr->datasamples,
 				    swapflag);
       msr->sampletype = 'i';
       break;
@@ -805,8 +840,8 @@ msr_unpack_data ( MSRecord *msr, int swapflag, int verbose )
       if ( verbose > 1 )
 	ms_log (1, "%s: Unpacking FLOAT-32 data samples\n", UNPACK_SRCNAME);
       
-      nsamples = msr_unpack_float_32 ((float *)dbuf, msr->samplecnt,
-				      msr->samplecnt, msr->datasamples,
+      nsamples = msr_unpack_float_32 ((float *)dbuf, (int)msr->samplecnt,
+				      (int)msr->samplecnt, msr->datasamples,
 				      swapflag);
       msr->sampletype = 'f';
       break;
@@ -815,8 +850,8 @@ msr_unpack_data ( MSRecord *msr, int swapflag, int verbose )
       if ( verbose > 1 )
 	ms_log (1, "%s: Unpacking FLOAT-64 data samples\n", UNPACK_SRCNAME);
       
-      nsamples = msr_unpack_float_64 ((double *)dbuf, msr->samplecnt,
-				      msr->samplecnt, msr->datasamples,
+      nsamples = msr_unpack_float_64 ((double *)dbuf, (int)msr->samplecnt,
+				      (int)msr->samplecnt, msr->datasamples,
 				      swapflag);
       msr->sampletype = 'd';
       break;
@@ -833,8 +868,8 @@ msr_unpack_data ( MSRecord *msr, int swapflag, int verbose )
       if ( verbose > 1 )
 	ms_log (1, "%s: Unpacking Steim-1 data frames\n", UNPACK_SRCNAME);
       
-      nsamples = msr_unpack_steim1 ((FRAME *)dbuf, datasize, msr->samplecnt,
-				    msr->samplecnt, msr->datasamples, diffbuff, 
+      nsamples = msr_unpack_steim1 ((FRAME *)dbuf, datasize, (int)msr->samplecnt,
+				    (int)msr->samplecnt, msr->datasamples, diffbuff, 
 				    &x0, &xn, swapflag, verbose);
       msr->sampletype = 'i';
       free (diffbuff);
@@ -852,8 +887,8 @@ msr_unpack_data ( MSRecord *msr, int swapflag, int verbose )
       if ( verbose > 1 )
 	ms_log (1, "%s: Unpacking Steim-2 data frames\n", UNPACK_SRCNAME);
       
-      nsamples = msr_unpack_steim2 ((FRAME *)dbuf, datasize, msr->samplecnt,
-				    msr->samplecnt, msr->datasamples, diffbuff,
+      nsamples = msr_unpack_steim2 ((FRAME *)dbuf, datasize, (int)msr->samplecnt,
+				    (int)msr->samplecnt, msr->datasamples, diffbuff,
 				    &x0, &xn, swapflag, verbose);
       msr->sampletype = 'i';
       free (diffbuff);
@@ -875,7 +910,7 @@ msr_unpack_data ( MSRecord *msr, int swapflag, int verbose )
 		    UNPACK_SRCNAME);
 	}
       
-      nsamples = msr_unpack_geoscope (dbuf, msr->samplecnt, msr->samplecnt,
+      nsamples = msr_unpack_geoscope (dbuf, (int)msr->samplecnt, (int)msr->samplecnt,
 				      msr->datasamples, msr->encoding, swapflag);
       msr->sampletype = 'f';
       break;
@@ -884,7 +919,7 @@ msr_unpack_data ( MSRecord *msr, int swapflag, int verbose )
       if ( verbose > 1 )
 	ms_log (1, "%s: Unpacking CDSN encoded data samples\n", UNPACK_SRCNAME);
       
-      nsamples = msr_unpack_cdsn ((int16_t *)dbuf, msr->samplecnt, msr->samplecnt,
+      nsamples = msr_unpack_cdsn ((int16_t *)dbuf, (int)msr->samplecnt, (int)msr->samplecnt,
 				  msr->datasamples, swapflag);
       msr->sampletype = 'i';
       break;
@@ -893,7 +928,7 @@ msr_unpack_data ( MSRecord *msr, int swapflag, int verbose )
       if ( verbose > 1 )
 	ms_log (1, "%s: Unpacking SRO encoded data samples\n", UNPACK_SRCNAME);
       
-      nsamples = msr_unpack_sro ((int16_t *)dbuf, msr->samplecnt, msr->samplecnt,
+      nsamples = msr_unpack_sro ((int16_t *)dbuf, (int)msr->samplecnt, (int)msr->samplecnt,
 				 msr->datasamples, swapflag);
       msr->sampletype = 'i';
       break;
@@ -902,7 +937,7 @@ msr_unpack_data ( MSRecord *msr, int swapflag, int verbose )
       if ( verbose > 1 )
 	ms_log (1, "%s: Unpacking DWWSSN encoded data samples\n", UNPACK_SRCNAME);
       
-      nsamples = msr_unpack_dwwssn ((int16_t *)dbuf, msr->samplecnt, msr->samplecnt,
+      nsamples = msr_unpack_dwwssn ((int16_t *)dbuf, (int)msr->samplecnt, (int)msr->samplecnt,
 				    msr->datasamples, swapflag);
       msr->sampletype = 'i';
       break;
@@ -921,7 +956,7 @@ msr_unpack_data ( MSRecord *msr, int swapflag, int verbose )
 /************************************************************************
  *  check_environment:
  *
- *  Check environment variables and set global variables approriately.
+ *  Check environment variables and set global variables appropriately.
  *  
  *  Return 0 on success and -1 on error.
  ************************************************************************/
